using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.IO;

using System.Net;
using System.Net.Sockets;

using kclass;

namespace igeterm
{
	public partial class form : Form
	{
		Microsoft.WindowsCE.Forms.InputPanel InputPanel;

		string IniPathName = "";

		private const int NON_PAGE_NO = -1;
		private const int TERM_PAGE_NO = 0;
		private const int TMP_PAGE_NO = 1;
		private const int EDIT_PAGE_NO = 2;
		private int selectEditPage = TERM_PAGE_NO;
		private encodeClass code = new encodeClass();

		private delegate void receiveMessageDelegate(byte[] data, int len);

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		public form()
		{
			InitializeComponent();

			if ((System.Environment.OSVersion.Platform == System.PlatformID.WinCE))
			{
				ceInit();
			}

			portBox.Items.Clear();
			foreach (string port in SerialPort.GetPortNames())
			{
				portBox.Items.Add(port);
			}
			string[] ports = SerialPort.GetPortNames();

			portBox.SelectedIndex = 0;
			speedBox.SelectedIndex = 7;
			rxCrBox.SelectedIndex = 0;
			txCrBox.SelectedIndex = 0;
			txCrBox.SelectedIndex = 0;
			codeBox.SelectedIndex = 0;
			dtrBox.Checked = serialPort.DtrEnable;
			rtsBox.Checked = serialPort.RtsEnable;

			string ExePath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName) + "\\";
			IniPathName = ExePath + System.IO.Path.GetFileNameWithoutExtension(System.Reflection.Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName) + ".ini";

			LoadSetupFile();
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// WM
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void ceInit()
		{
			InputPanel = new Microsoft.WindowsCE.Forms.InputPanel();
			InputPanel.EnabledChanged += new System.EventHandler(InputPanel_EnabledChanged);
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void InputPanel_EnabledChanged(object sender, EventArgs e)
		{
			if (InputPanel.Enabled)
			{
				tabControl.Dock = DockStyle.Top;
				tabControl.Height = Height - InputPanel.Bounds.Height;
			}
			else
			{
				tabControl.Dock = DockStyle.Fill;
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// L[(f[^M)
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void termBox_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (tabControl.SelectedIndex == 0)
			{
				byte[] buf = new byte[100];
				byte[] put = new byte[2];
				string str = "";
				str += e.KeyChar;
				buf = System.Text.Encoding.GetEncoding(codeBox.Text).GetBytes(str);


				try
				{
					if (buf[0] == '\r')
					{
						if (txCrBox.Text == "CR")				//CR(A)wH
						{
							put[0] = (byte)'\r';
							serialPort.Write(put, 0, 1);
						}
						else if (txCrBox.Text == "LF")			//LF(s)wH
						{
							put[0] = (byte)'\n';
							serialPort.Write(put, 0, 1);
						}
						else									//CR(A)LF(s)wH
						{
							put[0] = (byte)'\r';
							put[1] = (byte)'\n';
							serialPort.Write(put, 0, 2);
						}
					}
					else
					{
						serialPort.Write(buf, 0, buf.Length);
						//						for (int n = 0; n < buf.Length; n++)
						//						{
						//							serialPort.Write(buf, n, 1);
						//							System.Threading.Thread.Sleep(1000);
						//						}
					}
				}
				catch
				{
					MessageBox.Show("߂܂");
				}
			}
			e.Handled = true;
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// f[^MiʃXbhj
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
			byte[] buf = new byte[serialPort.BytesToRead];

			int len = serialPort.Read(buf, 0, buf.Length);
			if (len > 0)
			{
				this.Invoke(new receiveMessageDelegate(receiveMessage), new object[] { buf, len });
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// f[^M
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void receiveMessage(byte[] data, int len)
		{
			string str = "";
			string put = "";

			if (codeBox.Text == "us-ascii")
			{
				str = Encoding.ASCII.GetString(data, 0, len);
			}
			else if (codeBox.Text == "shift_jis")
			{
				str = code.GetStringSJIS(data, 0, len);
			}
			else if (codeBox.Text == "utf-8")
			{
				str = code.GetStringUTF8(data, 0, len);
			}
			else if (codeBox.Text == "big5")
			{
				str = code.GetStringBig5(data, 0, len);
			}
			else
			{
				str = System.Text.Encoding.GetEncoding(codeBox.Text).GetString(data, 0, len);
			}

			for (int n = 0; n < str.Length; n++)
			{
				switch (str[n])
				{
					case '\r':
						if (txCrBox.Text == "CR")				//CR(A)wH
						{
							put += "\r\n";
						}
						else if (txCrBox.Text == "LF")			//LF(s)wH
						{
						}
						else
						{
							put += str.Substring(n, 1);
						}
						break;

					case '\n':
						if (txCrBox.Text == "LF")				//LF(s)wH
						{
							put += "\r\n";
						}
						else if (txCrBox.Text == "CR")			//CR(A)wH
						{
						}
						else
						{
							put += str.Substring(n, 1);
						}
						break;

					case '\b':
						if (put.Length > 0)
						{
							if (put.Substring(put.Length - 1, 1) != "\n")
							{
								put.Remove(put.Length - 1, 1);
							}
						}
						else
						{
							if (termBox.Text.Length > 0)
							{
								termBox.SelectionStart = termBox.Text.Length - 1;
								termBox.SelectionLength = 1;
								if (termBox.SelectedText != "\n")
								{
									sendMesClass.cut(termBox.Handle);
								}
							}
						}
						break;

					case '\t':
						put += " ";
						break;

					default:
						put += str.Substring(n, 1);
						break;
				}
			}

			termBox.SelectionStart = termBox.Text.Length;
			termBox.SelectionLength = 0;
			sendMesClass.replase(termBox.Handle, put);

		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void dtrBox_CheckStateChanged(object sender, EventArgs e)
		{
			serialPort.DtrEnable = dtrBox.Checked;
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void checkBox_CheckStateChanged(object sender, EventArgs e)
		{
			serialPort.RtsEnable = rtsBox.Checked;
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// c؂ւ
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void setPage_Resize(object sender, EventArgs e)
		{
			//MessageBox.Show(this.Size.Height.ToString() + " " + setPage.Size.Height.ToString());setPage.Size.Height
			if (setPage.Size.Width > setPage.Size.Height)
			{
				setPanelA.Dock = System.Windows.Forms.DockStyle.Left;
				setPanelA.Width = setPage.Size.Width / 2;
			}
			else
			{
				setPanelA.Dock = System.Windows.Forms.DockStyle.Top;
				setPanelA.Height = setPage.Size.Height / 2;
			}

		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ^u؂ւ
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
		{
			int page = NON_PAGE_NO;

			string name = tabControl.TabPages[tabControl.SelectedIndex].Name;
			if (name == "termPage")
			{
				page = TERM_PAGE_NO;
			}
			else if (name == "tmpPage")
			{
				page = TMP_PAGE_NO;
			}
			else if (name == "editPage")
			{
				page = EDIT_PAGE_NO;
			}
			switch (page)
			{
				case TERM_PAGE_NO:
					undoMenu.Enabled = false;
					cutMenu.Enabled = false;
					clearMenu.Enabled = true;
					termBox.Focus();
					break;

				case TMP_PAGE_NO:
					undoMenu.Enabled = true;
					cutMenu.Enabled = true;
					clearMenu.Enabled = false;
					tmpBox.Focus();
					break;

				case EDIT_PAGE_NO:
					undoMenu.Enabled = true;
					cutMenu.Enabled = true;
					clearMenu.Enabled = false;
					editBox.Focus();
					break;

				default:
					undoMenu.Enabled = false;
					cutMenu.Enabled = false;
					clearMenu.Enabled = false;
					break;

			}
			selectEditPage = page;
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void termBox_KeyDown(object sender, KeyEventArgs e)
		{
			e.Handled = true;
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// J
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void openMenu_Click(object sender, EventArgs e)
		{
			if (openFileDialog.ShowDialog() == DialogResult.OK)
			{
				System.IO.StreamReader sr = new System.IO.StreamReader(openFileDialog.FileName, System.Text.Encoding.GetEncoding("shift_jis"));

				switch (selectEditPage)
				{
					case TERM_PAGE_NO:
						string str = sr.ReadToEnd();
						byte[] buf = new byte[str.Length + 1];
						buf = System.Text.Encoding.GetEncoding(codeBox.Text).GetBytes(str);
						try
						{
							serialPort.Write(buf, 0, buf.Length);
						}
						catch
						{
							MessageBox.Show("߂܂");
						}
						break;

					case TMP_PAGE_NO:
						tmpBox.Text = sr.ReadToEnd();
						break;

					case EDIT_PAGE_NO:
						editPage.Text = System.IO.Path.GetFileName(openFileDialog.FileName);
						editBox.Text = sr.ReadToEnd();
						break;
				}
				sr.Close();
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ۑ
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void saveMenu_Click(object sender, EventArgs e)
		{
			if (saveFileDialog.ShowDialog() == DialogResult.OK)
			{
				System.IO.StreamWriter sw = new System.IO.StreamWriter(saveFileDialog.FileName, false, System.Text.Encoding.GetEncoding("shift_jis"));

				switch (selectEditPage)
				{
					case TERM_PAGE_NO:
						sw.Write(termBox.Text);
						break;

					case TMP_PAGE_NO:
						sw.Write(tmpBox.Text);
						break;

					case EDIT_PAGE_NO:
						editPage.Text = System.IO.Path.GetFileName(saveFileDialog.FileName);
						sw.Write(editBox.Text);
						break;
				}
				sw.Close();
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ڑ
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void connectMenu_Click(object sender, EventArgs e)
		{
			if (connectMenu.Checked == false)
			{
				try
				{
					string port = portBox.Text;
					serialPort.PortName = port;
					serialPort.BaudRate = int.Parse(speedBox.Text);
					//serialPort.Encoding = System.Text.Encoding.GetEncoding(codeBox.Text);
					serialPort.Open();
					KeyPreview = true;
					connectMenu.Checked = true;
				}
				catch
				{
					MessageBox.Show(portBox.Text + "ɐڑł܂");
				}
			}
			else
			{
				KeyPreview = false;
				connectMenu.Checked = false;
				serialPort.Close();
			}

		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ɖ߂
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void undoMenu_Click(object sender, EventArgs e)
		{
			switch (selectEditPage)
			{
				case TMP_PAGE_NO:
					tmpBox.Undo();
					break;

				case EDIT_PAGE_NO:
					editBox.Undo();
					break;
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ؂
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void cutMenu_Click(object sender, EventArgs e)
		{
			switch (selectEditPage)
			{
				case TMP_PAGE_NO:
					if (tmpBox.Text != "")
					{
						Clipboard.SetDataObject(tmpBox.SelectedText);
						sendMesClass.cut(tmpBox.Handle);
					}
					break;

				case EDIT_PAGE_NO:
					if (editBox.Text != "")
					{
						Clipboard.SetDataObject(editBox.SelectedText);
						sendMesClass.cut(editBox.Handle);
					}
					break;
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// Rs[
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void copyMenu_Click(object sender, EventArgs e)
		{
			switch (selectEditPage)
			{
				case TERM_PAGE_NO:
					if (termBox.SelectedText != "")
					{
						Clipboard.SetDataObject(termBox.SelectedText);
					}
					break;

				case TMP_PAGE_NO:
					if (tmpBox.SelectedText != "")
					{
						Clipboard.SetDataObject(tmpBox.SelectedText);
					}
					break;

				case EDIT_PAGE_NO:
					if (editBox.SelectedText != "")
					{
						Clipboard.SetDataObject(editBox.SelectedText);
					}
					break;
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// \t
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void pasteMenu_Click(object sender, EventArgs e)
		{
			IDataObject obj = Clipboard.GetDataObject();
			if (obj.GetDataPresent(DataFormats.Text))
			{
				string str = (String)obj.GetData(DataFormats.Text);

				switch (selectEditPage)
				{
					case TERM_PAGE_NO:
						byte[] buf = new byte[str.Length + 1];
						buf = System.Text.Encoding.GetEncoding(codeBox.Text).GetBytes(str);
						try
						{
							serialPort.Write(buf, 0, buf.Length);
						}
						catch
						{
							MessageBox.Show("߂܂");
						}
						break;

					case TMP_PAGE_NO:
						sendMesClass.replase(tmpBox.Handle, str);
						break;

					case EDIT_PAGE_NO:
						sendMesClass.replase(editBox.Handle, str);
						break;
				}
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// SđI
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void allSelectMenu_Click(object sender, EventArgs e)
		{
			switch (selectEditPage)
			{
				case TERM_PAGE_NO:
					termBox.Focus();
					termBox.SelectAll();
					break;

				case TMP_PAGE_NO:
					tmpBox.Focus();
					tmpBox.SelectAll();
					break;

				case EDIT_PAGE_NO:
					editBox.Focus();
					editBox.SelectAll();
					break;
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// SăRs[
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void allCopyMenu_Click(object sender, EventArgs e)
		{
			switch (selectEditPage)
			{
				case TERM_PAGE_NO:
					if (termBox.Text != "")
					{
						Clipboard.SetDataObject(termBox.Text);
					}
					break;

				case TMP_PAGE_NO:
					if (tmpBox.Text != "")
					{
						Clipboard.SetDataObject(tmpBox.Text);
					}
					break;

				case EDIT_PAGE_NO:
					if (editBox.Text != "")
					{
						Clipboard.SetDataObject(editBox.Text);
					}
					break;
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ʏ
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void clearMenu_Click(object sender, EventArgs e)
		{
			switch (selectEditPage)
			{
				case TERM_PAGE_NO:
					termBox.Text = "";
					break;
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ݒt@C̓ǂݍ
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void LoadSetupFile()
		{
			try
			{
				string str;
				string[] array;

				StreamReader sr = new StreamReader(IniPathName, Encoding.UTF8);

				while ((str = sr.ReadLine()) != null)
				{
					array = str.Split(',', '=');

					if (array.Length == 2)
					{
						if (array[0].Trim() == "ʐM|[g")
						{
							portBox.Text = array[1].Trim();
						}
						else if (array[0].Trim() == "ʐMx")
						{
							speedBox.Text = array[1].Trim();
						}
						else if (array[0].Trim() == "ʐMMs")
						{
							txCrBox.Text = array[1].Trim();
						}
						else if (array[0].Trim() == "ʐMMs")
						{
							rxCrBox.Text = array[1].Trim();
						}
						else if (array[0].Trim() == "ʐMGR[h")
						{
							codeBox.Text = array[1].Trim();
						}
					}
				}
				sr.Close();

			}
			catch
			{
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ݒt@C̕ۑ
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void SaveSetupFile()
		{
			System.IO.StreamWriter sw = new System.IO.StreamWriter(IniPathName, false, System.Text.Encoding.UTF8);

			sw.WriteLine("ʐM|[g = " + portBox.Text);
			sw.WriteLine("ʐMx = " + speedBox.Text);
			sw.WriteLine("ʐMMs = " + txCrBox.Text);
			sw.WriteLine("ʐMMs = " + rxCrBox.Text);
			sw.WriteLine("ʐMGR[h = " + codeBox.Text);
			sw.Close();

		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// tH[ICxg
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void form_Closing(object sender, CancelEventArgs e)
		{
			SaveSetupFile();
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// ICxg
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		private void menuItem6_Click(object sender, EventArgs e)
		{
			Close();
		}

	}
}
