﻿using System;

using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;	//DllImportを使うための宣言

namespace kclass
{
	///-------------------------------------------------------------------------------------------------------------------------
	/// <summary>
	/// SendMessage関数を使用するためのクラス
	/// </summary>
	///-------------------------------------------------------------------------------------------------------------------------
	public class sendMesClass
	{
		/// <summary>
		/// 選択されているテキストをクリップボードにコピー（選択されているテキストは削除）
		/// </summary>
		public const int WM_CUT = 0x0300;

		/// <summary>
		/// 選択されているテキストをクリップボードへコピー
		/// </summary>
		public const int WM_COPY = 0x0301;

		/// <summary>
		/// クリップボードからコピー（選択されているテキストは削除）
		/// </summary>
		public const int WM_PASTE = 0x0302;

		/// <summary>
		/// 選択されているテキストを削除
		/// </summary>
		public const int WM_CLEAR = 0x0303;

		/// <summary>
		/// 選択されているテキストを置換
		/// </summary>
		public const int EM_REPLACESEL = 0x00C2;

		/// <summary>
		/// 
		/// </summary>
		public const int EM_LINEINDEX = 0x00BB;

		/// <summary>
		/// 
		/// </summary>
		public const int EM_LINEFROMCHAR = 0x00C9;

		/// <summary>
		/// wmフラグ
		/// </summary>
		private static bool wm = false;

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 静的コントラクタ
		/// </summary>
		///---------------------------------------------------------------------------------------------------------------------
		static sendMesClass()
		{
			if ((System.Environment.OSVersion.Platform == System.PlatformID.WinCE))
			{
				wm = true;
			}
			else
			{
				wm = false;
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// SendMessage関数を呼び出します。
		/// </summary>
		/// <param name="hWnd">送信先ウィンドウのハンドル</param>
		/// <param name="msg">メッセージ</param>
		///---------------------------------------------------------------------------------------------------------------------
		public static void send(IntPtr hWnd, int msg)
		{
			if (wm == true)
			{
				wmSendMessageClass.send(hWnd, msg);
			}
			else
			{
				pcSendMessageClass.send(hWnd, msg);
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// SendMessage関数を呼び出します。
		/// </summary>
		/// <param name="hWnd">送信先ウィンドウのハンドル</param>
		/// <param name="msg">メッセージ</param>
		/// <param name="wParam">最初のパラメータ</param>
		/// <param name="lParam">次のパラメータ</param>
		///---------------------------------------------------------------------------------------------------------------------
		public static int send(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam)
		{
			if (wm == true)
			{
				return (wmSendMessageClass.send(hWnd, msg, wParam, lParam));
			}
			else
			{
				return (pcSendMessageClass.send(hWnd, msg, wParam, lParam));
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 選択されているテキストをクリップボードにコピー（選択されているテキストは削除）
		/// </summary>
		/// <param name="hWnd">送信先ウィンドウのハンドル</param>
		///---------------------------------------------------------------------------------------------------------------------
		public static void cut(IntPtr hWnd)
		{
			if (wm == true)
			{
				wmSendMessageClass.send(hWnd, WM_CUT);
			}
			else
			{
				pcSendMessageClass.send(hWnd, WM_CUT);
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 選択されているテキストをクリップボードへコピー
		/// </summary>
		/// <param name="hWnd">送信先ウィンドウのハンドル</param>
		///---------------------------------------------------------------------------------------------------------------------
		public static void copy(IntPtr hWnd)
		{
			if (wm == true)
			{
				wmSendMessageClass.send(hWnd, WM_COPY);
			}
			else
			{
				pcSendMessageClass.send(hWnd, WM_COPY);
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// クリップボードからコピー（選択されているテキストは削除）
		/// </summary>
		/// <param name="hWnd">送信先ウィンドウのハンドル</param>
		///---------------------------------------------------------------------------------------------------------------------
		public static void paste(IntPtr hWnd)
		{
			if (wm == true)
			{
				wmSendMessageClass.send(hWnd, WM_PASTE);
			}
			else
			{
				pcSendMessageClass.send(hWnd, WM_PASTE);
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 選択されているテキストを削除
		/// </summary>
		/// <param name="hWnd">送信先ウィンドウのハンドル</param>
		///---------------------------------------------------------------------------------------------------------------------
		public static void clear(IntPtr hWnd)
		{
			if (wm == true)
			{
				wmSendMessageClass.send(hWnd, WM_PASTE);
			}
			else
			{
				pcSendMessageClass.send(hWnd, WM_PASTE);
			}
		}

		///---------------------------------------------------------------------------------------------------------------------
		/// <summary>
		/// 選択されているテキストを置換
		/// </summary>
		/// <param name="hWnd">送信先ウィンドウのハンドル</param>
		/// <param name="str">置換する文字列</param>
		///---------------------------------------------------------------------------------------------------------------------
		public static void replase(IntPtr hWnd, string str)
		{
			IntPtr wParam = new IntPtr(0);	//1:undo許可
			IntPtr lParam = System.Runtime.InteropServices.Marshal.StringToBSTR(str);

			if (wm == true)
			{
				wmSendMessageClass.send(hWnd, EM_REPLACESEL, wParam, lParam);
			}
			else
			{
				pcSendMessageClass.send(hWnd, EM_REPLACESEL, wParam, lParam);
			}

			System.Runtime.InteropServices.Marshal.FreeBSTR(lParam);
		}

	}

	///-------------------------------------------------------------------------------------------------------------------------
	/// <summary>
	/// PC用SendMessage関数を呼び出します。
	/// </summary>
	///-------------------------------------------------------------------------------------------------------------------------
	class pcSendMessageClass
	{
		[DllImport("user32.dll", CharSet = CharSet.Auto)]
		private static extern int SendMessage(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam);

		public static void send(IntPtr hWnd, int msg)
		{
			SendMessage(hWnd, msg, IntPtr.Zero, IntPtr.Zero);
		}

		public static int send(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam)
		{
			return (SendMessage(hWnd, msg, wParam, lParam));
		}
	}

	///-------------------------------------------------------------------------------------------------------------------------
	/// <summary>
	/// WM用SendMessage関数を呼び出します。
	/// </summary>
	///-------------------------------------------------------------------------------------------------------------------------
	class wmSendMessageClass
	{
		[DllImport("coredll.dll", CharSet = CharSet.Auto)]
		private static extern int SendMessage(IntPtr hWnd, int Msg, IntPtr wParam, IntPtr lParam);

		public static void send(IntPtr hWnd, int msg)
		{
			SendMessage(hWnd, msg, IntPtr.Zero, IntPtr.Zero);
		}

		public static int send(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam)
		{
			return (SendMessage(hWnd, msg, wParam, lParam));
		}
	}
}
